﻿# -*- coding: utf-8 -*-


"""
A Python bindings for MeCab DLL/shlib functions using ctypes

Version 2.0 by Yasushi MASUDA (ymasuda@ethercube.com)
Copyright Yasushi Masuda, 2002, 2003 All Rights Reserved.

Permission to use, copy, modify, and distribute this software and its
documentation for any purpose and without fee is hereby granted,
provided that above copyright notice and this permission notice
is attached with every copies or distributions.

YASUSHI MASUDA (THE AUTHOR HEREAFTER) DISCLAIMS ALL WARRANTIES WITH 
REGARD TO THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF 
MERCHANTABILITY AND FITNESS, IN NO EVENT THE AUTHOR SHALL BE LIABLE
FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER 
IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, 
ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF 
THIS SOFTWARE.
"""

from ctypes import *

# Specific type definitions for MeCab library
c_mecab_p = c_void_p

# Note: arglist specifications are omitted for *_sparse_tostr3() and
# *_next_tostr2() 
_MECAB_LIB_CATALOGUE = [
    ("mecab_new", c_mecab_p, [c_int, c_void_p]),
    ("mecab_new2", c_mecab_p, [c_char_p]),
    ("mecab_strerror", c_char_p, [c_mecab_p]),
    ("mecab_lock", c_int, [c_mecab_p]),
    ("mecab_unlock", c_int, [c_mecab_p]),
    ("mecab_sparse_tostr", c_char_p, [c_mecab_p, c_char_p]),
    ("mecab_sparse_tostr2", c_char_p, [c_mecab_p, c_char_p, c_uint]),
    #("mecab_sparse_tostr3", c_char_p, [c_mecab_p, c_char_p, c_uint,
    #                                   c_long, c_uint]),
    ("mecab_sparse_tostr3", None, []),
    ("mecab_nbest_sparse_tostr", c_char_p, [c_mecab_p, c_uint, c_char_p]),
    ("mecab_nbest_sparse_tostr2", c_char_p, [c_mecab_p, c_uint, c_char_p,
                                             c_uint]),
    #("mecab_nbest_sparse_tostr3", c_char_p, [c_mecab_p, c_uint, c_char_p,
    #                                         c_uint, c_long, c_uint]),
    ("mecab_nbest_sparse_tostr3", None, []),
    ("mecab_nbest_init", c_int, [c_mecab_p, c_char_p]),
    ("mecab_nbest_init2", c_int, [c_mecab_p, c_char_p, c_uint]),
    ("mecab_nbest_next_tostr", c_char_p, [c_mecab_p]),
    #("mecab_nbest_next_tostr2", c_char_p, [c_mecab_p, c_long, c_uint]),
    ("mecab_nbest_next_tostr2", None, []),
    ("mecab_nbest_next_tonode", None, []),
    ("mecab_destroy", None, [c_mecab_p]),
    ("mecab_do", None, []),
    ("mecab_make_dictionary", None, []), ]

def libmap(lib_path, func_catalogue, lib_type="cdll"):
    """Performs restype/argtypes mapping for each function in catalogue.
    """
    lib_type_map = {"cdll": cdll, "windll": windll, "oledll": oledll}
    lib = lib_type_map[lib_type].LoadLibrary(lib_path)
    for func_name, res_type, arg_types in func_catalogue:
        funcbody = getattr(lib, func_name)
        if res_type:
            funcbody.restype = res_type
        if arg_types:
            funcbody.argtypes = arg_types
    return lib

class MeCabError(StandardError):

    """
    Exception for MeCab DLL related error.
    
    """
    pass

class MeCabLib:

    """
    MeCabLib class, wraps C functions in MeCab DLL.

    """

    def __init__(self, arglist=[], libpath='libmecab.dll'):
        """Load MeCab library and initialize.
        """
        self._lib = libmap(libpath, _MECAB_LIB_CATALOGUE)
        self._mecab = self._lib.mecab_new2(' '.join(arglist))

    def __del__(self):
        """Unload MeCab library if exists.
        """
        self.destroy()
        del self._lib

    def reset(self, arglist):
        """Delete exisiting MeCab instance and create again.
        """
        if self._mecab:
            self.destroy()
        self._mecab = self._lib.mecab_new2(' '.join(arglist))

    def strerror(self):
        """Withdraw error description as string.
        """
        ret = self._lib.mecab_strerror(self._mecab)
        return ret

    def sparse_tostr(self, instr):
        """Analyze string and return parsing result.
        """
        ret = self._lib.mecab_sparse_tostr(self._mecab, instr)
        if ret==None:
            raise MeCabError()
        return ret

    def sparse_tostr2(self, instr, inlen):
        """Do sparse_tostr() with length specifier.
        """
        ret = self._lib.mecab_sparse_tostr2(self._mecab, instr, inlen)
        if ret==None:
            raise MeCabError()
        return ret

    def sparse_tostr3(self, instr, inlen, ostr_addr, olen):
        """Do sparse_tostr2() with given result buffer.

        Note: ostr_addr is the phisical address of the output buffer.

        """
        ret = self._lib.mecab_sparse_tostr3(self._mecab, instr, c_uint(inlen),
                                            ostr_addr, c_uint(olen))
        if ret==None:
            raise MeCabError()
        return ostr_addr

    def nbest_sparse_tostr(self, n, instr):
        """N-best version of sparse_tostr().
        """
        ret = self._lib.mecab_nbest_sparse_tostr(self._mecab, n, instr)
        if ret==None:
            raise MeCabError()
        return ret

    def nbest_sparse_tostr2(self, n, instr, inlen):
        """N-best version of sparse_tostr2().
        """
        ret = self._lib.mecab_nbest_sparse_tostr2(self._mecab, n, instr, inlen)
        if ret==None:
            raise MeCabError()
        return ret

    def nbest_sparse_tostr3(self, n, instr, inlen, ostr_addr, olen):
        """N-best version of sparse_tostr3().

        Note: ostr_addr is the phisical address of the output buffer.

        """
        ret = self._lib.mecab_nbest_sparse_tostr3(c_mecab_p(self._mecab),
                                                  instr, c_uint(inlen),
                                                  ostr_addr, c_uint(olen))
        if ret==None:
            raise MeCabError()
        return ostr

    def nbest_init(self, instr):
        """Initialize N-best session for given string.
        """
        ret = self._lib.mecab_nbest_init(self._mecab, instr)
        if ret==0:
            raise MeCabError()
        return ret
        
    def nbest_init2(self, instr, inlen):
        """Do nbest_init() with length specifier.
        """
        ret = self._lib.mecab_nbest_init2(self._mecab, instr, inlen)
        if ret==0:
            raise MeCabError()
        return ret

    def nbest_next_tostr(self):
        """Withdraw a subsequent result from current N-best session.
        """
        ret = self._lib.mecab_nbest_next_tostr(self._mecab)
        if ret==None:
            raise MeCabError
        return ret        
    
    def nbest_next_tostr2(self, ostr_addr, olen):
        """Do nbest_next_tostr() with given result buffer.

        Note: ostr_addr is the phisical address of the output buffer.

        """
        ostr = c_buffer.from_address(ostr_addr)
        ret = self._lib.mecab_nbest_next_tostr(self._mecab, ostr, olen)
        if ret==None:
            raise MeCabError
        return ostr

    def lock(self):
        """Mutex lock this instance.
        """
        ret = self._lib.mecab_lock(self._mecab)
        return ret

    def unlock(self):
        """Unlock mutex from this instance.
        """
        ret = self._lib.mecab_unlock(self._mecab)
        return ret

    def destroy(self):
        """Destroy current mecab object.
        """
        self._lib.mecab_destroy(self._mecab)


if __name__=="__main__":
    instring = "健が直美に貸した本を浩にわたした．"
    m = MeCabLib(arglist=['-a'], libpath='e:/Apps/mecab/bin/libmecab.dll')
    print m.sparse_tostr(instring)
    print "-" * 60
    instring = "きしゃのきしゃがきしゃできしゃした"
    m.nbest_init(instring)
    print m.nbest_next_tostr()
    print "-" * 60
    print m.nbest_next_tostr()


